/*
*--------------------------------------------------------------------
* This software is contributed or developed by KYOCERA Corporation.
* (C) 2017 KYOCERA Corporation
*--------------------------------------------------------------------
*/
package com.vzw.basicphone.speech.tts;

import android.content.ContentValues;
import android.content.Context;
import android.net.Uri;
import android.os.Bundle;
import android.speech.tts.TextToSpeech;
import android.speech.tts.UtteranceProgressListener;

public class VzwTextToSpeech extends UtteranceProgressListener {

    public static final int SUCCESS = 0x00000000;
    public static final int ERROR   = 0xFFFFFFFF;

    private static final Uri URI_STATUS_PROVIDER =
            Uri.parse("content://com.google.android.marvin.talkback.providers.StatusProvider");
    private static final String KEY_FORCE_INACTIVE = "FORCE_INACTIVE";

    private final TextToSpeech mTextToSpeech;
    private UtteranceProgressListener mListener;
    private Context mContext;

    public VzwTextToSpeech(Context context, OnInitListener listener) {
        mContext = context;
        mTextToSpeech = new TextToSpeech(mContext,listener);
        ContentValues values = new ContentValues();
        values.put(KEY_FORCE_INACTIVE,true);
        mContext.getContentResolver().update(URI_STATUS_PROVIDER,values,null,null);
    }

    public int setUtteranceProgressListener(UtteranceProgressListener listener){
        mListener = listener;
        return mTextToSpeech.setOnUtteranceProgressListener(this);
    }

    public int speak(String text){
        return  mTextToSpeech.speak(text,TextToSpeech.QUEUE_FLUSH,new Bundle(),text);
    }

    public int stop(){
        return mTextToSpeech.stop();
    }

    public boolean isSpeaking(){
        return mTextToSpeech.isSpeaking();
    }

    public void shutdown(){
        mTextToSpeech.shutdown();
        ContentValues values = new ContentValues();
        values.put(KEY_FORCE_INACTIVE,false);
        mContext.getContentResolver().update(URI_STATUS_PROVIDER,values,null,null);
    }

    @Override
    public void onStart(String s) {
        if(mListener != null){
            mListener.onStart(s);
        }
    }

    @Override
    public void onDone(String s) {
        if(mListener != null){
            mListener.onDone(s);
        }
    }

    @Override
    public void onError(String s) {
        onError(s,TextToSpeech.ERROR);
    }

    @Override
    public void onError(String utteranceId, int errorCode) {
        if(mListener!= null){
            mListener.onError(utteranceId,errorCode);
        }
    }

    @Override
    public void onStop(String utteranceId, boolean interrupted) {
        if(mListener != null){
            mListener.onStop(utteranceId,interrupted);
        }
    }

    public interface UtteranceProgressListener{
        void onDone(String utteranceId);
        void onError(String utteranceId, int errorCode);
        void onStart(String utteranceId);
        void onStop(String utteranceId, boolean interrupted);
    }


    public interface OnInitListener extends TextToSpeech.OnInitListener{
        void onInit(int status);
    }
}
